#  include <boost/units/systems/si.hpp>
using namespace boost::units;
#  include <blitz/array.h>
using namespace blitz;
#include <boost/program_options.hpp>
namespace po = boost::program_options;

/// error reporting
#define error_macro(msg) { cerr << "-- error: " << msg << endl; throw exception(); }
#define warning_macro(msg) { cerr << "-- warning: " << msg << endl; }

#include "model.hpp"

int main(int ac, char* av[])
{
  try
  {
    /// options list
    po::options_description desc("options");
    desc.add_options()
      ("help", "print this message")
      ("bits", po::value<int>(), "floating point bits: 32, 64, 80, 128, ...")
      ("adv", po::value<string>(), "advection scheme: upstream, leapfrog, mpdata")
      ("dom", po::value<string>(), "domain: serial, openmp, threads")
      ("nk", po::value<int>(), "number of subdomains (nx/nk must be int)")
      ("nx", po::value<int>(), "number of grid points")
      ("nt", po::value<int>(), "number of timesteps")
      ("dt", po::value<string>(), "timestep length [s]")
      ("dx", po::value<string>(), "gridbox length [m]")
      ("u", po::value<string>(), "velocity [m/s]")
    ;
    po::variables_map vm;
    po::store(po::parse_command_line(ac, av, desc), vm);
    po::notify(vm);

    /// --help or no argument case
    if (vm.count("help") || ac == 1)
    {
      cerr << desc << endl;
      exit(EXIT_FAILURE);
    }

    /// --bits (floating point precision choice)
    if (!vm.count("bits")) error_macro("floating point precision not specified (--bits option)")
    int bits = vm["bits"].as<int>();
    if (sizeof(float) * 8 == bits) model<float>(vm);
    //else if (sizeof(double) * 8 == bits) model<double>(vm);
    //else if (sizeof(long double) * 8 == bits) model<long double>(vm);
    else error_macro("unsupported number of bits (" << bits << ")")
  }
  catch (exception &e)
  {
    cerr << "-- exception cought: " << e.what() << endl;
    exit(EXIT_FAILURE);
  }
}
