#ifndef DOM_SERIAL_HPP
#  define DOM_SERIAL_HPP

#  include "dom.hpp"
#  include "adv.hpp"

template <class unit, typename real_t>
class dom_serial : public dom<unit, real_t>
{
  private: Array<quantity<unit, real_t>, 1> **psi;
  private: adv<unit, real_t> *fllbck, *advsch;
  private: Range *i;

  public: dom_serial(adv<unit, real_t> *fllbck, adv<unit, real_t> *advsch, int i_min, int i_max)
    : fllbck(fllbck), advsch(advsch)
  {
    // memory allocation
    int halo = (advsch->stencil_extent() - 1) / 2;
    psi = new Array<quantity<unit, real_t>, 1>*[advsch->time_levels()]; 
    for (int n=0; n < advsch->time_levels(); ++n) 
      psi[n] = new Array<quantity<unit, real_t>, 1>(Range(i_min - halo, i_max + halo)); 
    i = new Range(i_min, i_max);
  
    // initial condition (FIXME! - now it's always: |...... )
    if (i_min == 0) (*psi[0])(Range(0)) = 1;

    // periodic boundary
    this->hook_neighbours(this, this);
  }

  public: ~dom_serial()
  {
    for (int n=0; n < advsch->time_levels(); ++n) delete psi[n];
    delete[] psi;
    delete i;
  }

  public: void output(int n, bool endrec = true)
  {
    cout << (*psi[n])(*i);
    if (endrec) cout << endl << endl;
  }

  // FIXME: support for halo != 0
  private: quantity<unit, real_t> *rght_edge(int n) { return psi[n]->dataZero() + i->last(); }
  private: quantity<unit, real_t> *left_edge(int n) { return psi[n]->dataZero() + i->first(); }

  public: void fill_halos(int n)
  {
    // FIXME: support for halo != 0
    (*psi[n])(Range(i->first() - 1)) = *(this->left_halo_data(n));
    (*psi[n])(Range(i->last()  + 1)) = *(this->rght_halo_data(n));
  }

  public: void advect(adv<unit, real_t> *a, int n, quantity<si::dimensionless, real_t> &C, int s)
  {
    *psi[n+1] = *psi[0];
    a->op(psi, *i, n, C, s);
  }

  public: void cycle_arrays(const int n)
  {
    switch (advsch->time_levels())
    {
      case 2: cycleArrays(*psi[n], *psi[n+1]); break;
      case 3: cycleArrays(*psi[n-1], *psi[n], *psi[n+1]); break;
      default: assert(false);
    }
  }

  public: void integ_loop(int nt, quantity<si::dimensionless, real_t> &C) 
  {
    int n;
    for (int t = 0; t < nt; ++t)
    {   
      adv<unit, real_t> *a;
      bool fallback = this->choose_an(&a, &n, t, advsch, fllbck);    
      output(n);
      for (int s = 1; s <= a->num_steps(); ++s)
      {   
        fill_halos(n);
        advect(a, n, C, s); 
        if (!fallback) cycle_arrays(n);
      } // s
    } // t
    output(n);
  }

};
#endif
